@@ -21,12 +21,13 @@ module Bundler
         @dependency_names = []
         @allow_remote = false
         @allow_cached = false
-        @caches = [Bundler.app_cache, *Bundler.rubygems.gem_cache]
+        @caches = [cache_path, *Bundler.rubygems.gem_cache]
 
         Array(options["remotes"] || []).reverse_each {|r| add_remote(r) }
       end
 
       def remote!
+        @specs = nil
         @allow_remote = true
       end
 
@@ -123,31 +124,34 @@ module Bundler
           Bundler.ui.confirm message
 
           path = cached_gem(spec)
-          if Bundler.requires_sudo?
+          if requires_sudo?
             install_path = Bundler.tmp(spec.full_name)
             bin_path     = install_path.join("bin")
           else
-            install_path = Bundler.rubygems.gem_dir
+            install_path = rubygems_dir
             bin_path     = Bundler.system_bindir
           end
 
           installed_spec = nil
           Bundler.rubygems.preserve_paths do
-            installed_spec = Bundler::RubyGemsGemInstaller.new(
+            installed_spec = Bundler::RubyGemsGemInstaller.at(
               path,
               :install_dir         => install_path.to_s,
               :bin_dir             => bin_path.to_s,
               :ignore_dependencies => true,
               :wrappers            => true,
-              :env_shebang         => true
+              :env_shebang         => true,
+              :build_args          => opts[:build_args],
+              :bundler_expected_checksum => spec.respond_to?(:checksum) && spec.checksum
             ).install
           end
+          spec.full_gem_path = installed_spec.full_gem_path
 
           # SUDO HAX
-          if Bundler.requires_sudo?
+          if requires_sudo?
             Bundler.rubygems.repository_subdirectories.each do |name|
               src = File.join(install_path, name, "*")
-              dst = File.join(Bundler.rubygems.gem_dir, name)
+              dst = File.join(rubygems_dir, name)
               if name == "extensions" && Dir.glob(src).any?
                 src = File.join(src, "*/*")
                 ext_src = Dir.glob(src).first
@@ -173,7 +177,7 @@ module Bundler
 
         spec.post_install_message
       ensure
-        Bundler.rm_rf(install_path) if Bundler.requires_sudo?
+        Bundler.rm_rf(install_path) if requires_sudo?
       end
 
       def cache(spec, custom_path = nil)
@@ -247,7 +251,7 @@ module Bundler
       end
 
       def loaded_from(spec)
-        "#{Bundler.rubygems.gem_dir}/specifications/#{spec.full_name}.gemspec"
+        "#{rubygems_dir}/specifications/#{spec.full_name}.gemspec"
       end
 
       def cached_gem(spec)
@@ -289,6 +293,10 @@ module Bundler
             next if spec.name == "bundler" && spec.version.to_s != VERSION
             have_bundler = true if spec.name == "bundler"
             spec.source = self
+            if Bundler.rubygems.spec_missing_extensions?(spec, false)
+              Bundler.ui.debug "Source #{self} is ignoring #{spec} because it is missing extensions"
+              next
+            end
             idx << spec
           end
 
@@ -315,11 +323,14 @@ module Bundler
         @cached_specs ||= begin
           idx = installed_specs.dup
 
-          path = Bundler.app_cache
-          Dir["#{path}/*.gem"].each do |gemfile|
+          Dir["#{cache_path}/*.gem"].each do |gemfile|
             next if gemfile =~ /^bundler\-[\d\.]+?\.gem/
             s ||= Bundler.rubygems.spec_from_gem(gemfile)
             s.source = self
+            if Bundler.rubygems.spec_missing_extensions?(s, false)
+              Bundler.ui.debug "Source #{self} is ignoring #{s} because it is missing extensions"
+              next
+            end
             idx << s
           end
         end
@@ -343,7 +354,7 @@ module Bundler
 
           # because ensuring we have all the gems we need involves downloading
           # the gemspecs of those gems, if the non-api sites contain more than
-          # about 100 gems, we just treat all sites as non-api for speed.
+          # about 100 gems, we treat all sites as non-api for speed.
           allow_api = idx.size < API_REQUEST_LIMIT && dependency_names.size < API_REQUEST_LIMIT
           Bundler.ui.debug "Need to query more than #{API_REQUEST_LIMIT} gems." \
             " Downloading full index instead..." unless allow_api
@@ -401,16 +412,16 @@ module Bundler
         uri = spec.remote.uri
         spec.fetch_platform
 
-        download_path = Bundler.requires_sudo? ? Bundler.tmp(spec.full_name) : Bundler.rubygems.gem_dir
-        gem_path = "#{Bundler.rubygems.gem_dir}/cache/#{spec.full_name}.gem"
+        download_path = requires_sudo? ? Bundler.tmp(spec.full_name) : rubygems_dir
+        gem_path = "#{rubygems_dir}/cache/#{spec.full_name}.gem"
 
         SharedHelpers.filesystem_access("#{download_path}/cache") do |p|
           FileUtils.mkdir_p(p)
         end
         Bundler.rubygems.download_gem(spec, uri, download_path)
 
-        if Bundler.requires_sudo?
-          SharedHelpers.filesystem_access("#{Bundler.rubygems.gem_dir}/cache") do |p|
+        if requires_sudo?
+          SharedHelpers.filesystem_access("#{rubygems_dir}/cache") do |p|
             Bundler.mkdir_p(p)
           end
           Bundler.sudo "mv #{download_path}/cache/#{spec.full_name}.gem #{gem_path}"
@@ -418,7 +429,7 @@ module Bundler
 
         gem_path
       ensure
-        Bundler.rm_rf(download_path) if Bundler.requires_sudo?
+        Bundler.rm_rf(download_path) if requires_sudo?
       end
 
       def builtin_gem?(spec)
@@ -432,6 +443,18 @@ module Bundler
       def installed?(spec)
         installed_specs[spec].any?
       end
+
+      def requires_sudo?
+        Bundler.requires_sudo?
+      end
+
+      def rubygems_dir
+        Bundler.rubygems.gem_dir
+      end
+
+      def cache_path
+        Bundler.app_cache
+      end
     end
   end
 end
